/* 
 * command from file
 * 2004.11.26-
 * Time-stamp: <2005-08-03 23:25:15 dimm>
 */
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "finder.h"
#include "environ.h"

/* start command-check timer */
void start_comm_timer(void) {
  int t = DEF_TIMER_COMM;
  timertag_comm = gtk_timeout_add(t, (GtkFunction)timeout_comm_check, NULL);
  #ifdef DEBUG
  printf("# setting command check interval:%d msec.\n", t);
  #endif
}

/* stop command-check timer */
void stop_comm_timer(void) {
  gtk_timeout_remove(timertag_comm);
  timertag_comm = -1;
}

/* command check */
void timeout_comm_check(gpointer pointer) {
  FILE *fp;
  char buf[256];
  int ret;
  int comm = 0;

  #ifdef DEBUG
  printf("# timeout_comm_check() called.\n");
  #endif

  checkenv(); /* in command.c */

  if ((fp = fopen(COMMANDFILE, "r")) != NULL) {
    fgets(buf, 256, fp);
    #ifdef DEBUG
    printf("# received command:%s",buf);
    #endif
    if (!(strncmp(buf, "start", 5)) || !(strncmp(buf, "START", 5))) {
      comm = 1;
    }
    if (!(strncmp(buf, "quit", 4)) || !(strncmp(buf, "QUIT", 4))) {
      comm = 2;
    }
    if (!(strncmp(buf, "stop", 4)) || !(strncmp(buf, "STOP", 4))) {
      comm = 3;
    }
    fclose(fp);

    sprintf(buf, "/bin/rm -f %s\n", COMMANDFILE);
    #ifdef DEBUG
    printf("# execute %s", buf);
    #endif
    system(buf);
  }

  switch(comm) {
  case 1:
    printf("# timeout_comm_check(): received start command.\n");
    mseemon.execflg = 1; /* set to activate procedure in seemon2.c */
    mseemon.is_auto = 1;
    //    stop_comm_timer();

    /* intialize log file */
    setoutputfile();

    /* socket connection */
    if (fd_socket == -1) {
      if ((ret = connect_socket(SOCKET_SERVER)) == EXIT_SUCCESS) {
	printf("connected to %s.\n", SOCKET_SERVER);
      } else {
	if (fout != NULL) {
	  fprintf(fout, "unable to connect to %s", SOCKET_SERVER);
	  fd_socket=-1;
	  printf("# unable to connect to %s", SOCKET_SERVER);
	  return;
	}
      }
    }

    if (domestatus == 1) {
      if (rainflg == RAIN_FALSE &&
	  humidity > 0 && humidity < mseemon.hum_limit) {
	/* environment check passed. open the dome. */
	if (fout != NULL) {
	  output_ctime(fout);
	  fprintf(fout, " opening the dome.\n");
	}
	#ifdef DEBUG
	printf("# opening the dome.\n");
	#endif
	dome_open();
	if (strcmp(sockhost,"dimm1")==0) {
	  /* DIMM2dome */
	  printf("# waiting 450 sec.\n");
	  sleep(450);
	} else {
	  /* DIMM1dome */
	  printf("# waiting 15 sec.\n");
	  sleep(15);
	}
	domestatus = read_dome_status(); /* in socket_client.c */
      }
    }

    if (domestatus == 3) {
      /* AC on */
      ret = ccd_on();
      if (ret == DOME_ERR) {
	printf("# failed in CCD power-on.\n");
	return;
      }

      /* wait 15 sec for secure start-up */
      printf("# wainting 15 sec.\n");
      sleep(15);

      auto_start_capture(); /* in cap_auto.c */
      pos_sw = 1; /* start coord output */
    }
    break;
  case 2:
    printf("# timeout_comm_check(): received quit command.\n");
    stop_comm_timer();
    auto_stop_capture(); /* in cap_auto.c */
    //    mseemon.loopflg = 0;
    break;
  case 3:
    printf("# timeout_comm_check(): received stop command.\n");
    auto_stop_capture(); /* in cap_auto.c */
    break;
  }

  printf("# finished comm_check.\n");

}

/* check rain / humidity */
/* 20050127 */
void checkenv(void) {
  rainflg = chk_raindrop();
  humidity = chk_humidity();
  domestatus = read_dome_status(); /* in socket_client.c */

  if ((rainflg == RAIN_TRUE) || (humidity >= mseemon.hum_limit)) {
    if (domestatus > 1) {
      printf("# checkenv: rain:%d humidity:%d dome:%d.\n", rainflg, humidity, domestatus);
      if (fout != NULL) {
	output_ctime(fout);
	fprintf(fout, "# checkenv: rain:%d humidity:%d dome:%d.\n", rainflg, humidity, domestatus);
      }
      dome_close(); /* in socket_client.c */
      if (mseemon.is_auto) {
	auto_stop_capture();
	/* notify seemon2 */
	write_finder_coords(); /* in socket_client.c */
      } else {
	callback_hide_window_cap (NULL, (gpointer)window_capture);
	/* notify seemon2 */
	write_finder_coords(); /* in socket_client.c */
      }
    }
  }
}
