/* 2002.10.26-
 * finder.h
 * based on seemon2.h
 *
 * Time-stamp: <2005-08-03 23:30:44 dimm>
 */
#include <stdio.h>

#include "P_.h"
#include "circum.h"

#ifndef PI
#define	PI		3.141592653589793
#endif

#define IMG_HGT 480
#define IMG_WID 640
//#define RECT_HGT 50
//#define RECT_WID 80
#define RECT_HGT 30
#define RECT_WID 30
#define VERSION "0.7.7"
#define PPMPATH "/mnt/ramdisk/seemon2/tmp1000000.ppm"
//#define PPMPATH "/mnt/ramdisk/seemon2/tmp000000.ppm"
//#define POSFILE "/mnt/ramdisk/seemon2/pos.txt"
//#define DOMESTATFILE "/mnt/ramdisk/seemon2/domestat.txt"
//#define PPMPATH "/usr/local/share/seemon2/tmp1000000.ppm"
#define POSFILE "/usr/local/share/seemon2/pos.txt"
#define DOMESTATFILE "/usr/local/share/seemon2/domestat.txt"
#define LOGPATH "/usr/local/share/seemon2/finder_log"
#define COMMANDFILE "/usr/local/share/seemon2/command.txt"
#define N_CALC_R0 100

#define BUFFER_SIZE 20

/* codes for exiting 
 * added finder v0.6.7 20041104 */
#define EXIT_NORMAL 0
#define EXIT_ERROR  -2

/* smmothing size for peak detection.
 * added from seemon2 ver.0.2.6
 */
#define SMOOTH_SIZE 3
#define CG_RECT 10

/* for cap_window.c; default time interval of dv read-out */
//#define DEF_TIMER 2000
#define DEF_TIMER 5000
/* command-check timer */
#define DEF_TIMER_COMM 60000
/* max of error counter */
#define MAX_ERROR 10

/* parameters depending on hardware
 * you should change these for new hardware.
 */
#define DIAM 0.05      /* diameter of apertures in unit of metre */
#define SEP  0.144     /* separation of apertures in unit of metre */
#define PIXSCALE 0.499 /* pixel scale in unit of arcsec/pix */
#define LAMBDA 0.55e-6 /* observing wavelength in unit of metre */

/* humidity warning threshold */
#define HUM_LIMIT 90

/* error from finder_comm */
#define	DOME_ERR -1

/* pixel coords */
typedef struct {
  int cx;
  int cy;
} pixcoord;

/* coord in double */
typedef struct {
  double x;
  double y;
} dcoord;

/* for time.c */
typedef struct {
  int year;
  int month;
  int day;
  int hour;
  int min;
  int sec;
} myTime;

/* observation site info */
typedef struct {
  double slon;
  double slat;
  unsigned char lt;
  char name[256];
} Site;

/* seemon2 global settings */
typedef struct {
  int objnum; /* number of objects in object list */
  int selnum; /* selected object number */
  int time_reload; /* interval for reload dv in msec */
  int time_sky;    /* interval for setting sky */
  int cnt_sky;     /* counter for setting sky */
  int skyval;      /* background sky value */
  float skysig;    /* background sky stddev, added finder v0.6.6 20041019 iwata */
  int peakval[2];  /* stellar peak values */
  char flg_reload; /* flg for reloading */
  char msg[256];   /* for goto command to telescope */
  double tel_ra;   /* current ra in radian */
  double tel_dc;   /* current dc in radian */
  int tel_step;    /* motion step in arcsec */
  double trg_ra;   /* target ra  in radian */
  double trg_dc;   /* target dec in radian */
  char dvfilepath[256];   /* path of .ppm file (converted from dv image) */
  int tel_stat;    /* telescope status. 0) disconnect 1) connected 2) pointing 3) pointed */
  pixcoord star_center[2];
  pixcoord star_rect[2];
  pixcoord peak[2];    /* star peaks, added ver. 0.2.6 */
  dcoord cg[2];
  dcoord oldcg[2];
  dcoord diffcg[N_CALC_R0];
  dcoord r0;
  dcoord fwhm;
  int is_capturing; /* capturing=1, otherwise 0 */
  int is_tracking;  /* tracking=1; not tracking=0 */
  int is_recording; /* recording=1 */
  int motionflg;    /* checker for large cg motion. 0 if motion is large */
  myTime ctime;     /* (current) time */
  int tel_dir[4];   /* info of telescope handbox buttons */
  double image_dist; /* expected distance of two stellar images (in pixels)
		      * used for tracking check */
  int is_restart;   /* 1 if finder is started with option -r, otherwise 0, added v0.6.8 20041106 iwata */
  int is_auto;      /* 1:auto 0:manual */
  //  int loopflg;      /* v0.7.0 */
  int execflg;      /* v0.7.0 */
  int hum_limit;    /* v0.7.5 */
} Seemon;


/* definition of functions */
/* cap_window.c */
int init_cap_window(GtkWidget *window_cap);
void callback_open_window_cap (GtkWidget *widget, gpointer data);
void timeout_reload_image(gpointer pointer);
void starttimer(void);
void stoptimer(void);
int recreate_cap_window(void); /* destroy and re-create capture window v0.6.3 2004.02.21 */

/* cap_auto.c v0.7.0 2004.11.22 iwata */
int auto_start_capture(void);
void auto_stop_capture(void);
void timeout_reload_auto(gpointer pointer);

/* callback.c */
void callback( GtkWidget *widget, gpointer data );
gint delete_event( GtkWidget *widget, GdkEvent *event, gpointer data );
void callback_open_window (GtkWidget *widget, gpointer data);
void callback_hide_window (GtkWidget *widget, gpointer data);
void callback_close_window(GtkWidget *widget, gpointer data);
void callback_open_setting(GtkWidget *widget);
void mydialog(char *str); /* generic dialog */

/* obj.c */
int read_edb(char* fname);
GtkListStore* setup_objlist(int num);
int update_objazel(int num, GtkListStore *store);

/* menu.c */
GtkWidget* init_main_menu(GtkWidget *window);

/* find_star.c */
int get_star(void);
int search_peak(int n);
int get_bg(void); /* finder v0.6.6 20041019 */

/* in prefs.c */
int init_setup_dialog(GtkWidget *window_setup);

/* domeio.c */
//int check_dome_status(void);
void callback_dome_status(GtkWidget *widget, gpointer data);
//int check_dome_status_file(void);
void callback_dome_open(GtkWidget *widget, gpointer data);
void callback_dome_close(GtkWidget *widget, gpointer data);
//int dome_open_proc(void);
//int dome_close_proc(void);

/* in outputlog.c v0.6.3 */
int setoutputfile (void);
void closeoutputfile(void);
void output_ctime(FILE *fp);

/* in time.c */
int get_time(myTime *ttime);
int get_localtime(myTime *ttime);

/* in socket_client.c */
int connect_socket(char* hostname);
int disconnect_socket(void);
int read_dome_status(void);
int dome_open(void);
int dome_close(void);
int ccd_on(void);
int ccd_off(void);
int reset_sio(void);

/* in command.c */
void start_comm_timer(void);
void stop_comm_timer(void);
void timeout_comm_check(gpointer pointer);
void checkenv(void); /* 20050127 */

/* in config.c
 *  added v0.7.4 20050720 */
int read_conf(char* configfile);

/******* 
 * global variables
 *******/
guchar* graybuf; /* pointer for gray-scale image buffer (read from DV) */
int wid, hgt;    /* image size */
char flg_set_rect[2]; /* flags to judge rectangles are set */

/* variables used in get_cg.c */
int *pImRect[2]; /* pointers for rectangular image buffer */
double bgthr; /* background subtraction threshold */
double cgfactor; /* cg adjust factor, imported from seemon1 */
int n_calc_r0;	 /* number of data points for calculating r0 */
dcoord Sigma, Mean;
int NSample;
int marker_mode; /* 1:marking mode 0:normal */
int is_marked;   /* 1:mark exists */
pixcoord mark;
int is_star_found; /* 1:star found */
pixcoord starpos;

/* for fifo */
int pos_sw; /* 1 for output coords */

/* dome status*/
int domestatus; /* 1:close 2:unknown 3:open */
/* rain flag */
int rainflg;     /* 1:rain 0:no rain */
int humidity;   /* humidity */

/* generic global settings */
Seemon mseemon;

/* site info */
Site msite;

/* finder program status 
 * added v0.6.3 2004.02.22 */
int prg_status; /* working:1, normal termination:0, error:-2 */
int prg_cnt;    /* error counter */

/* pointer for output log file */
FILE *fout;

/* socket v0.7.0 */
#define SOCKET_SERVER "dimm2"
int fd_socket;
char sockhost[32];

int netconnect; /* network connection, 1=connected v0.7.4 20050722 */
int standalone; /* RS-232C connected to dome controller=0, standalone=1 20050726 */

/* widgets */
GtkWidget *ent1, *ent2, *ent3, *ent4, *ent5, *ent6;  /* used in prefs.c */
GtkWidget *ent_intv;
GtkWidget *g_image_area1; /* captured image area, used in cap_window.c */
GdkGC     *g_pen1;        /* graphic context, used in cap_window.c */
GtkWidget *ent_r0;        /* current r0 in cap_window.c */
GtkWidget *toggle_cstep;  /* current selected toggle button for tel. step. used in telctl_window.c */
gint timertag;            /* timer tag, used in cap_window.c */
gint timertag_comm;       /* timer tag for command check */
//GtkWidget *btn1;          /* used in cap_window.c */
GtkWidget *window_capture;  /* used in cap_window.c; added on v.0.6.2 */
