/* 
 * Time-stamp: <2005-02-02 17:56:28 dimm>
 * finder_comm.c
 * 
 *  for communication bet. finder and seemon2
 *
 * 2003.08.17
 * 2003.09.05
 * 2003.09.06 not using fifo
 * 2004.11.18 accept shutter speed commands
 * 2004.11.23 fork version
 */
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/stat.h>

#include "dome.h"
#include "finder_comm.h"

int main(void) {
  FILE *fp;
  //  int ret, ct, cx, cy, speed;
  int ret, speed, sw;
  char buf[BUFFER_SIZE], line[BUFFER_SIZE];
  char subbuf, *e;
  char sp_char[5];
  pid_t pid;

  int fd1, fd2, len;
  struct sockaddr_in    saddr;
  struct sockaddr_in    caddr;

  fx = 0; fy = 0; fpeak = 0;
  rainflg = 1; humidity = 100;

  /* init sio */
  //  fd=init_sio();
  //  if(fd<0)
  //    return DOME_ERR;

  /* check dome status */
  domestatus = dome_status(); /* in domeio.c */


  /* create socket */
  if ((fd1 = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket");
    exit(EXIT_FAILURE);
  }
  bzero((char *)&saddr, sizeof(saddr));
  saddr.sin_family = AF_INET;
  saddr.sin_addr.s_addr=INADDR_ANY;
  saddr.sin_port=htons(PORT);

  if (bind(fd1, (struct sockaddr *)&saddr, sizeof(saddr)) < 0) {
    perror("bind");
    exit(EXIT_FAILURE);
  }

  /* listen */
  if (listen(fd1, 5) < 0) {
    perror("listen");
    exit(EXIT_FAILURE);
  }

  signal(SIGCHLD, SIG_IGN);

  while(1) {
    #ifdef DEBUG
    printf("# finder_comm: waiting for a connection.\n");
    #endif

    len = sizeof(caddr);
    if ((fd2 = accept(fd1, (struct sockaddr *)&caddr, &len)) < 0) {
      perror("accept");
      exit(EXIT_FAILURE);
    }

    //    close(fd1);
    printf("connection established.\n");

    fd_socket = fd2;

    pid = fork();

    switch(pid) {
    case -1:
      exit(EXIT_FAILURE);
    case 0: /* child */
      while((ret=read(fd_socket, buf, sizeof(buf))) > 0) {
	subbuf = buf[0];
	if ((ret = strcmp(buf, "quit")) == 0) {
	  break;
	}

	sw = 0;

	/* dome open */
	if ((ret = strcmp(buf, "dopen")) == 0) sw = 1;

	/* dome close */
	if ((ret = strcmp(buf, "dclose")) == 0) sw = 2;

	/* dome status */
	if ((ret = strcmp(buf, "dome")) == 0) sw = 3;

	/* CCD power-on */
	if ((ret = strcmp(buf, "ccdon")) == 0) sw = 4;

	/* CCD power-off */
	if ((ret = strcmp(buf, "ccdoff")) == 0) sw = 5;

	/* shutter speed, v0.6.9 */
	if (subbuf == 's') sw =6;

	/* read finder coords, v0.7.0 */
	if (subbuf == 'f') sw = 7;

	/* flush sio v0.7.0 */
	if ((ret = strcmp(buf, "reset")) == 0) sw = 8;

	/* start finder v0.7.2 */
	if ((ret = strcmp(buf, "fstart")) == 0) sw = 9;

	/* set time to start finder v0.7.2 */
	if ((ret = strcmp(buf, "timeset")) == 0) sw = 10;

	switch(sw) {
	case 1: /* dome open */
	  printf("# received dome open command.\n");
	  ret = dome_open_proc();
	  break;
	case 2: /* dome close */
	  printf("# received dome close command.\n");
	  ret = dome_close_proc();
	  break;
	case 3: /* dome status */
	  printf("# received dome status command.\n");
	  ret = dome_status_proc();
	  break;
	case 4: /* ccd power on */
	  printf("# received CCD power-on command.\n");
	  ret = ccd_on();
	  break;
	case 5: /* ccd power off */
	  printf("# received CCD power-off command.\n");
	  ret = ccd_off();
	  break;
	case 6: /* shutter speed */
	  sp_char[0] = buf[1];
	  sp_char[1] = buf[2];
	  sp_char[2] = buf[3];
	  sp_char[3] = buf[4];
	  sp_char[4] = '\0';
	  speed = (int)strtol(sp_char, &e, 16);
	  printf("# received a command %s to set shutter speed 0x%s(=%d) usec.\n", buf, sp_char, speed);
	  ret = set_shutter_speed(sp_char);
	  break;
	case 7: /* read finder coords from socket, write to file*/
	  sscanf(buf, "f %d %d %d %d %d", &fx, &fy, &fpeak, &rainflg, &humidity);
	  #ifdef DEBUG
	  printf("# read from socket: fx:%d fy:%d rain:%d hum:%d.\n", fx, fy, rainflg, humidity);
	  #endif

	  if ((fp = fopen(POSFILE, "w"))==NULL) {
	    printf("# failed in opening %s.\n", POSFILE);
	  } else {
	    domestatus = dome_status();
	    fprintf(fp, "%d %d %d %d %d %d\n", fx, fy, fpeak, rainflg, humidity, domestatus);
	  }
	  fclose(fp);
	  break;
	case 8:
	  ret = flush_sio();
	  break;
	default: /* read finder coords, rain, hum from file, then output via socket */
	  if ((fp = fopen(POSFILE, "r"))==NULL) {
	    printf("# failed in opening %s.\n", POSFILE);
	  } else {
	    fgets(line, BUFFER_SIZE, fp);
	    printf("#read from %s:%s\n", POSFILE, line);
	  }
	  fclose(fp);

	  write(fd_socket, line, sizeof(line));
	  break;
	case 9: /* put start command */
	  printf("# received a finder-start command.\n");
	  fp = fopen(COMMANDFILE, "w");
	  fprintf(fp, "start");
	  fclose(fp);
	  break;
	case 10:
	  /* set start command using scr.checktime02finder.pl */
	  system("/usr/local/bin/scr.checktime02finder.pl");
	  break;
	}
      }

      close(fd_socket);
      exit(EXIT_SUCCESS);
      break;
    default: /* parent */
      close(fd_socket);
      break;
    }
  }

}



/* procedure for opening the dome */
int dome_open_proc(void) {
  FILE *fp;
  int ret, status;
  //  int ret, ct = 0, status;
  //  char line[20];

  status = dome_status();
  /*
  if ((fp = fopen(DOMESTATFILE, "r")) == NULL) {
    printf("failed opening %s.\n", DOMESTATFILE);
    return(EXIT_FAILURE);
  }

  while (fgets(line, 20, fp)) {
    if (ct == 0) {
      sscanf(line, "%d", &status);
      if (status == DOME_STAT_OPEN) {
      printf("dome is already opened.\n");
      }
      ct++;
    }
  }

  fclose(fp);
  */
  if (status == DOME_STAT_OPEN) {
      printf("dome is already opened.\n");
  } else {
    ret = dome_open();

    if (ret == DOME_OK) {
      printf("succeeded in opening the dome.\n");
      /* write to file */
      if ((fp = fopen(DOMESTATFILE, "w")) == NULL) {
	printf("failed in opening %s.\n", DOMESTATFILE);
	return(EXIT_FAILURE);
      }
      fprintf(fp, "%d\n", DOME_STAT_OPEN);
      fclose(fp);
    } else {
      printf("failed in opening the dome.\n");
      return(EXIT_FAILURE);
    }
  }

  return(EXIT_SUCCESS);
}

/* procedure for closing the dome */
int dome_close_proc(void) {
  FILE *fp;
  int ret, ct = 0, status;
  char line[20];

  if ((fp = fopen(DOMESTATFILE, "r")) == NULL) {
    printf("failed opening %s.\n", DOMESTATFILE);
    return(EXIT_FAILURE);
  }

  while (fgets(line, 20, fp)) {
    if (ct == 0) {
      sscanf(line, "%d", &status);
      if (status == DOME_STAT_CLOSE) {
	printf("dome is already closed.\n");
      }
      ct++;
    }
  }

  fclose(fp);

  ret = dome_close();

  if (ret == DOME_OK) {
    printf("succeeded in closing the dome.\n");
    /* write to file */
    if ((fp = fopen(DOMESTATFILE, "w")) == NULL) {
      printf("failed in opening %s.\n", DOMESTATFILE);
      return(EXIT_FAILURE);
    }
    fprintf(fp, "%d\n", DOME_STAT_CLOSE);
    fclose(fp);
  } else {
    printf("failed in closing the dome.\n");
    return(EXIT_FAILURE);
  }

  return(EXIT_SUCCESS);
}


/* procedure for checking dome status */
int dome_status_proc(void) {
  char line[5];

  domestatus = dome_status(); /* in dome.c */

  sprintf(line, "%d ", domestatus);

  write(fd_socket, line, sizeof(line));
  #ifdef DEBUG
  printf("# dome_status_proc(): sent:%s\n", line);
  #endif

  return(EXIT_SUCCESS);
}
